#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# Do NOT edit this system file by hand -- use git.  See "URL to git source" below.
#
# Author:        $Id: Thomas R. Stevenson <aa0026@wayne.edu> $
#
# Last Changed:  $Date: Mon Feb 20 16:57:47 2023 -0500 $
#
# URL to git source: $URL: git@git.wayne.edu:ECS_Projects/ECSpylibs.git $
#

# Futuristic implementation of commands.  Must be first from/import
# command.

from __future__ import print_function

# Python Standard libraries.

import autologging
import logging
import psutil

from autologging import logged, traced


@traced
@logged
class ReapChildren:
    """Locate all children processes, if any, and kill them."""

    def __init__(self) -> None:
        """Nothing to setup."""

    def reap_children(self, timeout: object = 3) -> None:
        """Locate all children processes, if any, and kill them."""

        def on_terminate(proc: object) -> object:
            """Log any process killed by reapChildren."""

            self.__log.warning("Process %s terminated with exit code %s.", proc, proc.returncode)

        processes = psutil.Process().children(recursive=True)

        for process in processes:
            self.__log.warning("Process %s survived drive.quit(); trying SIGTERM!", process)
            process.terminate()

        gone, alive = psutil.wait_procs(processes, timeout=timeout, callback=on_terminate)

        if not alive:
            for process in alive:
                self.__log.error("Process %s survived SIGTERM; trying SIGKILL!", process)
                process.kill()

            gone, alive = psutil.wait_procs(alive, timeout=timeout, callback=on_terminate)

            if not alive:
                for process in alive:
                    self.__log.critical("Process %s survived SIGKILL; giving up!", process)



#!/usr/bin/env python3.6
# -*- coding: utf-8 -*-
#
# Do NOT edit this system file by hand -- use git.  See "URL to git source" below.
#
# Author:        $Id: Thomas R. Stevenson <aa0026@wayne.edu> $
#
# Last Changed:  $Date: Fri Feb 17 12:47:46 2023 -0500 $
#
# URL to git source: $URL: git@git.wayne.edu:ECS_Projects/ECSpylibs.git $
#

# Futuristic implementation of commands.  Must be first from/import
# command.

from __future__ import print_function

# Python Standard libraries.

import autologging
import logging
import mailbox
import sys
import zipfile

from autologging import logged, traced
from os.path import abspath, basename, dirname, isabs, isdir, isfile, ismount, join, normpath


@traced
@logged
class ParseEmail:
    """ blah """

    def __init__(self, mailbox_file: object, data_dir: object) -> None:
        """ blah """

        self.data = None
        self.file_name = None
        self.part_type = None
        self.msg_subject = None
        self.msg = None
        self.init = True

        if not isfile(mailbox_file) or not isdir(data_dir):
            self.init = False
            self.__log.warning(f"One or more parameters are missing or invalid.")

            if not isfile(mailbox_file):
                self.__log.critical(f"File '{mailbox_file}' does not exist.")

            if not isdir(data_dir):
                self.__log.critical(f"Directory '{data_dir}' does not exist.")

            self.__log.warning(f"Aborting call to ParseEmail.__init__({mailbox_file}, {data_dir}).")
        else:
            self.mailbox_file = mailbox_file
            self.data_dir = data_dir
            self.mbox = None

    def open_mbox(self) -> object:
        """ blah """

        self.__log.debug(f"Opening mailbox file '{self.mailbox_file}'.")
        self.mbox = mailbox.mbox(self.mailbox_file)
        return True

    def get_attachments(self, subject: object, attachment_file: object) -> object:
        """ blah """

        self.__log.info(f"Get attachments with subject '{subject}' and file name '{attachment_file}'.")

        for self.key in self.mbox.keys():
            self.__log.info(f"Processing message {self.key}.")

            self.msg = self.mbox[self.key]
            self.msg_subject = self.msg["Subject"]

            self.__log.info(f"Message {self.key} with subject '{self.msg_subject}'.")

            if self.msg_subject == subject:
                self.__log.info(f"Message {self.key} has good subject '{subject}'.")

                for self.part in self.msg.walk():
                    self.__log.info(f"Walking through message {self.key}.")

                    self.part_type = self.part.get_content_disposition()

                    self.__log.info(f"Message {self.key} has content disposition '{self.part_type}'.")

                    if self.part_type == "attachment":
                        self.__log.info(f"Message {self.key} has good content disposition 'attachment'.")

                        self.file_name = self.part.get_filename()

                        self.__log.info(f"Message {self.key} has file name '{self.file_name}'")

                        if self.file_name == attachment_file:
                            self.__log.info(f"Message {self.key} has good file name '{attachment_file}'.")

                            self.__log.info(f"Extract attachment file name '{attachment_file}' from message {self.key}.")

                            self.data = self.part.get_payload(decode=True)

                            self.__log.info(
                                f"Storage attachment file '{attachment_file}' into directory '{self.data_dir}'.")

                            with open(normpath(join(self.data_dir, self.file_name)), "wb") as fp:
                                fp.write(self.data)

                            self.__log.info(
                                f"Unzip attachment file name '{attachment_file}' into directory '{self.data_dir}'")

                            with zipfile.ZipFile(normpath(join(self.data_dir, self.file_name)), "r") as zip_ref:
                                zip_ref.extractall(self.data_dir)

                            self.__log.info(
                                f"Attachment file '{attachment_file}' unzipped into directory '{self.data_dir}'.")

                        else:
                            self.__log.warning(f"Invalid attachment file name '{self.file_name}'.")
                    else:
                        self.__log.info(f"Skipping content disposition type '{self.part_type}'.")
            else:
                self.__log.warning(f"Skipping email with subject '{self.msg_subject}'.")

            self.mbox.lock()
            self.mbox.discard(self.key)
            self.mbox.flush()
            self.mbox.unlock()

        return True

    def close_mbox(self) -> object:
        """ blah """

        self.__log.info(f"Close mailbox file '{self.mailbox_file}'.")

        if self.mbox:
            self.mbox.close()
            self.mbox = None

        return True



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Do NOT edit this system file by hand -- use git.  See "URL to git source" below.
#
# Author:        $Id: Thomas R. Stevenson <aa0026@wayne.edu> $
#
# Last Changed:  $Date: Fri Feb 17 12:47:46 2023 -0500 $
#
# URL to git source: $URL: git@git.wayne.edu:ECS_Projects/ECSpylibs.git $
#

# Futuristic implementation of commands.  Must be first from/import
# command.

from __future__ import print_function

# Python Standard libraries.

import autologging
import logging
import os
import re
import shutil
import tempfile

from autologging import logged, traced
from os.path import abspath, basename, dirname, isabs, isdir, isfile, ismount, join, normpath


@traced
@logged
class CleanupTempDirs:
    """Locate all children processes, if any, and kill them."""

    def __init__(self) -> None:
        """Setup initial variables."""

        self.delete_tmp_dir = None
        self.tmp_dir_pat = re.compile("^scoped_dir[0-9]+_[0-9]+$")
        self.user_temp_directory = tempfile.gettempdir()

    def cleanup_temp_dirs(self) -> None:
        """Locate all old temp files that were not deleted from previous runs and delete them."""

        if self.user_temp_directory:
            if isdir(self.user_temp_directory):
                self.__log.debug('User Temp Directory is located at "%s".', self.user_temp_directory)
                self.__log.debug('Looking for temp directories which match pattern "^scoped_dir[0-9]+_[0-9]+$".')

                for self.tmp_root, self.tmp_dirs, self.tmp_files in os.walk(self.user_temp_directory):
                    for self.tmp_dir in self.tmp_dirs:
                        self.delete_tmp_dir = join(self.tmp_root, self.tmp_dir)

                        if self.tmp_dir_pat.match(self.tmp_dir):
                            self.__log.warning('Found a temp dir that should have been deleted: "%s".',
                                               self.delete_tmp_dir)

                            # Try to remove the directory.  Just continue if the remove fails.
                            try:
                                self.__log.info('Delete directory "%s".', self.delete_tmp_dir)
                                shutil.rmtree(self.delete_tmp_dir)
                                self.__log.info('Directory "%s" was deleted.', self.delete_tmp_dir)
                            except OSError as e:
                                self.__log.error('Error: %s - %s.', e.filename, e.strerror)
                                self.__log.exception('Exception: "%s".', e)
                            except Exception as e:
                                self.__log.critical('Fatal error occurred while removing directory "%s".',
                                                    self.delete_tmp_dir)
                                self.__log.exception('Exception: "%s".', e)
            else:
                self.__log.error('Variable for User Temp Directory was not a directory "%s".', self.user_temp_directory)
        else:
            self.__log.error('Variable for User Temp Directory was not defined.')



#!/usr/bin/env python3.6
# -*- coding: utf-8 -*-
#
# Do NOT edit this system file by hand -- use git.  See "URL to git source" below.
#
# Author:        $Id: Thomas R. Stevenson <aa0026@wayne.edu> $
#
# Last Changed:  $Date: Fri Feb 17 12:47:46 2023 -0500 $
#
# URL to git source: $URL: git@git.wayne.edu:ECS_Projects/ECSpylibs.git $
#

# Futuristic implementation of commands.  Must be first from/import
# command.

from __future__ import print_function

# Python Standard libraries.

import autologging
import logging
import smtplib

from autologging import logged, traced
from email.mime.application import MIMEApplication
from email.mime.multipart import MIMEMultipart
from email.mime.text import MIMEText
from email.utils import COMMASPACE, formatdate
from pathlib import Path


@traced
@logged
class SendEmail:
    """ blah """

    def __init__(self) -> None:
        """ blah """

        self.part = None
        self.msg = None
        self.smtp = None
        self.f = None


    def send_email(self,
                   from_id: object,
                   to_list: object,
                   cc_list: object,
                   subject: object,
                   text: object,
                   text_type: object = "plain",
                   files: object = None,
                   server: object = "127.0.0.1"
                   ) -> None:
        """ blah """

        assert isinstance(to_list, list)
        if cc_list:
            assert isinstance(cc_list, list)
            send_list = to_list + cc_list
        else:
            send_list = to_list

        self.__log.debug(f"fromID    : '{from_id}'.")
        self.__log.debug(f"toList    : '{to_list}'.")
        self.__log.debug(f"ccList    : '{cc_list}'.")
        self.__log.debug(f"send_list : '{send_list}'.")
        self.__log.debug(f"subject   : '{subject}'.")
        self.__log.debug(f"text      : '{text}'.")
        self.__log.debug(f"textType  : '{text_type}'.")
        if files:
            self.__log.debug(f"files : '{files}'.")

        self.msg = MIMEMultipart()

        self.msg['From'] = from_id
        self.msg['To'] = COMMASPACE.join(to_list)
        if cc_list:
            self.msg['Cc'] = COMMASPACE.join(cc_list)
        self.msg['Date'] = formatdate(localtime=True)
        self.msg['Subject'] = subject

        self.msg.attach(MIMEText(text, text_type))

        for self.f in files or []:
            try:
                self.f = Path(self.f).resolve(strict=True)
            except FileNotFoundError as e:
                self.__log.exception(f"FileNotFoundError: {e=}")
            except Exception as e:
                self.__log.exception(f"Exception: {e=}")
                sys.exit(5)

            with open(self.f, "rb") as self.file:
                self.part = MIMEApplication(
                    self.file.read(),
                    Name=self.f.name
                )

            # After the file is closed
            self.part['Content-Disposition'] = f'attachment; filename="{self.f.name}"'
            self.msg.attach(self.part)

        self.__log.info(f"Sending email.")

        self.smtp = smtplib.SMTP(server)
        self.smtp.sendmail(from_id, send_list, self.msg.as_string())
        self.smtp.close()

        self.__log.info(f"Email sent.")



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Do NOT edit this system file by hand -- use git.  See "URL to git source" below.
#
# Author:        $Id: Thomas R. Stevenson <aa0026@wayne.edu> $
#
# Last Changed:  $Date: Tue Feb 21 12:10:36 2023 -0500 $
#
# URL to git source: $URL: git@git.wayne.edu:ECS_Projects/ECSpylibs.git $
#

"""
Doc String
"""

# Python Standard libraries.

import sys
from pathlib import Path
import logging
import os
import re
import time
from socket import gethostname
from symbol import argument
from dataclasses import dataclass
from ecspylibs.configfile import ConfigFile

import autologging
import yaml
from autologging import logged, traced
from lxml import etree


@traced
@logged
@dataclass
class InitSetup:
    """Initialize the environment using command line parameters and init file."""
    arguments: dict
    default_arguments: dict
    config: object = None
    section: object = None

    def __post_init__(self):
        """Setup InitSetup"""

        if self.config is not None:
            if self.config in self.arguments and self.arguments[self.config] is not None:
                self.config = self.arguments[self.config]
            elif self.config in self.default_arguments and self.default_arguments[self.config] is not None:
                self.config = self.default_arguments[self.config]
            else:
                self.config = False
        else:
            self.config = False

        if self.config:
            # try:
            #     self.config = Path(self.config).resolve(strict=True)
            # except FileNotFoundError as e:
            #     print(f"\n: Can't find configuration file {self.config}\n", file=sys.stderr)
            #     print(f"\nFileNotFoundError: {e=}\n", file=sys.stderr)
            #     sys.exit(1)
            # except Exception as e:
            #     print(f"\nException: {e=}\n", file=sys.stderr)
            #     sys.exit(2)

            # with open(self.config, 'r') as file:
            #     config_data = yaml.safe_load(file)

            config_file = ConfigFile(self.config)
            config_data = config_file.read

            if 'config' not in config_data:
                print(f"\nError: Can't find dictionary 'config' in file '{self.config}'. Aborting!\n", file=sys.stderr)
                sys.exit(3)

            config_len = len(config_data['config'])
            section_number = 1

            if self.section and self.section in self.arguments and self.arguments[self.section]:
                if isinstance(self.arguments[self.section], int) or self.arguments[self.section].isnumeric():
                    section_number = int(self.arguments[self.section])
                else:
                    print(f"\nArgument section number '{section_number}' is missing or invalid", file=sys.stderr)
                    print(f"for file '{self.config}'.", file=sys.stderr)
                    print(f"Aborting!\n", file=sys.stderr)
                    sys.exit(4)

            elif 'section' in config_data:
                if isinstance(config_data['section'], int):
                    section_number = int(config_data['section'])

            if section_number > config_len or section_number <= 0:
                print(f"\nSection number '{section_number}' is out of range for file", file=sys.stderr)
                print(f"'{self.config}'.", file=sys.stderr)
                print(f"Aborting!\n", file=sys.stderr)
                sys.exit(5)

            self.data = config_data['config'][section_number - 1]

            for i in self.arguments:
                if self.arguments[i] is None:
                    if i[2:] in self.data:
                        self.arguments[i] = self.data[i[2:]]
                    elif i in self.default_arguments:
                        self.arguments[i] = self.default_arguments[i]
        else:
            for i in self.arguments:
                if self.arguments[i] is None:
                    if i in self.default_arguments:
                        self.arguments[i] = self.default_arguments[i]



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# Do NOT edit this system file by hand -- use git.  See "URL to git source" below.
#
# Author:        $Id: Thomas R. Stevenson <aa0026@wayne.edu> $
#
# Last Changed:  $Date: Fri Feb 17 12:47:46 2023 -0500 $
#
# URL to git source: $URL: git@git.wayne.edu:ECS_Projects/ECSpylibs.git $
#

# Futuristic implementation of commands.  Must be first from/import
# command.

from __future__ import print_function

# Home grown libraries.

from ecspylibs.buildfunctionlist import BuildFunctionList

# Python Standard libraries.

import autologging
import logging
import sys

from autologging import logged, traced
from lxml import etree


@traced
@logged
class ParseXML(BuildFunctionList):
    """Class to parse an XML file into its parts."""

    def __init__(self, xml_file: object, conf_version: object, command_prefix: object = "function") -> None:
        """Setup the ParseXML environment."""

        self.root = None
        self.katalon_commands = None
        self.tree = None
        self.xml_file = xml_file
        self.conf_version = conf_version
        self.xml_file_errors = False
        self.program_errors = False

        super().__init__(command_prefix)

    def parse_katalon_xml_headers(self) -> bool:
        """Parse Katalon XML file headers."""

        try:
            self.tree = etree.parse(self.xml_file)
        except Exception as e:
            self.__log.error("Exception error while parsing file '%s'.", self.xml_file)
            self.__log.exception("Exception: '%s'", e)
            self.xml_file_errors = True
            self.program_errors = True
            return False

        self.root = self.tree.getroot()

        if self.root.tag != "TestCase":
            self.__log.error("Missing or invalid root tag line: '%s'", self.root.tag)
            return False

        self.katalon_commands = self.tree.xpath("./*")
        return True

    def parse_xml_headers(self) -> bool:
        """Parse Monitor XML file headers."""

        try:
            self.tree = etree.parse(self.xml_file)
        except Exception as e:
            self.__log.error("Exception error while parsing file '%s'.", self.xml_file)
            self.__log.exception("Exception: '%s'", e)
            self.xml_file_errors = True
            self.program_errors = True
            return False

        self.root = self.tree.getroot()

    def process_katalon_root(self, command_prefix: object = None) -> bool:
        """Process the root entry in the Katalon XML file."""

        for self.katalon_command in self.katalon_commands:
            if self.program_errors or self.xml_file_errors:
                if self.program_errors:
                    self.__log.error("Internal errors processing Katalon Root.  Aborting!")
                else:
                    self.__log.warning("External errors processing Katalon Root.  Aborting!")
                return False

            if not self.call(self.katalon_command.tag, self.katalon_command, command_prefix):
                return False

        return True

    def process_children(self, args: object, command_prefix: object = None) -> bool:
        """Process all the children entries in the Katalon or Monitor XML files."""

        for self.cmd in args.xpath("./*"):
            if self.program_errors or self.xml_file_errors:
                if self.program_errors:
                    self.__log.error("Internal errors processing Children.  Aborting!")
                else:
                    self.__log.warning("External errors processing Children.  Aborting!")
                return False

            if not self.call(self.cmd.tag, self.cmd, command_prefix):
                return False

        return True



from pkg_resources import get_distribution, DistributionNotFound

try:
    # Change here if project is renamed and does not equal the package name
    dist_name = __package__
    __version__ = get_distribution(dist_name).version
except DistributionNotFound:  # pragma: no cover
    # Can't unit test this for now since I do not know how to temporarily
    # remove a distribution during runtime. Help. Also, is this even
    # worth it?
    __version__ = "unknown"



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# Do NOT edit this system file by hand -- use git.  See "URL to git source" below.
#
# Author:        $Id: Thomas R. Stevenson <aa0026@wayne.edu> $
#
# Last Changed:  $Date: Fri Feb 17 12:47:46 2023 -0500 $
#
# URL to git source: $URL: git@git.wayne.edu:ECS_Projects/ECSpylibs.git $
#

# Futuristic implementation of commands.  Must be first from/import
# command.

from __future__ import print_function

# Python Standard libraries.

import autologging
import logging
import sys

from autologging import logged, traced
from inspect import isfunction


@traced
@logged
class BuildFunctionList:
    """Validate and call an internal function given the function name as a string."""

    def __init__(self, default_command_prefix: object = "function") -> None:
        """Set default function name prefix."""

        self.default_command_prefix = default_command_prefix
        self.program_errors = False
        self.xml_file_errors = False

    def call(self, function_name: object, function_args: object, command_prefix: object = None) -> object:
        """Validate and call an internal funtion given the function name as a string."""

        if command_prefix == None:
            self.function_name = "%s__%s" % (self.default_command_prefix, function_name,)
        else:
            self.function_name = "%s__%s" % (command_prefix, function_name,)

        self.__log.debug("Validating function : %s", self.function_name)

        if self.__class__.__dict__.get(self.function_name):

            self.__log.debug("%s : %s", self.function_name, self.__class__.__dict__[self.function_name])
            if isfunction(self.__class__.__dict__[self.function_name]):

                self.__log.info("Calling function : %s", self.function_name)
                return self.__class__.__dict__[self.function_name](self, function_args)

            else:

                self.__log.error("Requested function name is invalid : '{self.functionName}'")
                print(f"Requested function name is invalid : '{self.functionNam}'", file=sys.strerr)
                self.xml_file_errors = True

        else:

            self.__log.error("BuildFunctionList.call called with invalid function name %s in class %s.",
                             self.function_name, self.__class__)
            self.__log.warning("Add the following line to %s: def %s(self, args): pass", self.__class__,
                               self.function_name)
            self.__log.warning("Or fix the XML file entry that called '%s'.", self.function_name)
            self.xml_file_errors = True



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Do NOT edit this system file by hand -- use git.  See "URL to git source" below.
#
# Author:        $Id: Thomas R. Stevenson <aa0026@wayne.edu> $
#
# Last Changed:  $Date: Tue Feb 21 12:10:36 2023 -0500 $
#
# URL to git source: $URL: git@git.wayne.edu:ECS_Projects/ECSpylibs.git $
#

"""
Doc String
"""

import sys
import json
import os

from dataclasses import dataclass
from json.decoder import JSONDecodeError
from pathlib import Path, PosixPath

import pandas as pd
import toml
import yaml

from lxml import etree
from lxml.etree import XMLSyntaxError
from toml.decoder import TomlDecodeError
from xlrd.biffh import XLRDError
from yaml.parser import ParserError
from yaml.scanner import ScannerError

@dataclass
class ConfigFile:
    """Read a configuration file and return its data."""

    file: str = None

    def __post_init__(self):
        """Setup ConfigFile."""

        self.suffix_types = {
            ".conf": self.toml_read,
            ".json": self.json_read,
            ".toml": self.toml_read,
            ".xlsx": self.xlsx_read,
            ".xml": self.xml_read,
            ".yaml": self.yaml_read,
            ".yml": self.yaml_read,
        }

        if type(self.file) is list:
            for tmp in self.file:
                if type(tmp) is PosixPath and tmp.is_file():
                    self.file = tmp.resolve(strict=False)
                    break
                elif type(tmp) is str:
                    tmp = Path(tmp).resolve(strict=False)
                    if tmp.is_file():
                        self.file = tmp.resolve(strict=False)
                        break
            else:
                raise FileNotFoundError

        if type(self.file) is str and self.file:
            self.file = Path(self.file).resolve(strict=False)

        if type(self.file) is PosixPath and self.file.is_file():
            try:
                self.file = self.file.resolve(strict=True)
            except FileNotFoundError as e:
                print(f"\nMissing or invalid configuration file '{self.file}'.\n", file=sys.stderr)
                raise e
            except Exception as e:
                print(f"\nException error processing file '{self.file}'.\n", file=sys.stderr)
                raise e
        else:
            raise TypeError("Parameter must be an existing file name.")

        if self.file.suffix not in self.suffix_types:
            print(f"\nFile '{self.file}' has an unknown suffix type of '{self.file.suffix}'.\n", file=sys.stderr)
            raise NotImplementedError

        self.suffix = self.file.suffix[1:]

    @property
    def read(self) -> object:

        if self.file.suffix in self.suffix_types:
            try:
                return self.suffix_types[self.file.suffix]()
            except PermissionError as e:
                print(f"\nPermission error reading file '{self.file}'.\n", file=sys.stderr)
                raise e
            except UnicodeDecodeError as e:
                print(f"\nUnicode decode error while parsing file '{self.file}'.\n", file=sys.stderr)
                raise e
            except Exception as e:
                print(f"\nException error processing file '{self.file}'.\n", file=sys.stderr)
                raise e
        else:
            print(f"\nFile '{self.file}' has an unknown suffix type of '{self.file.suffix}'.\n", file=sys.stderr)
            raise NotImplementedError

    def json_read(self) -> object:
        try:
            with open(self.file, 'r') as open_file:
                return json.load(open_file)
        except JSONDecodeError as e:
            print(f"\nJSON decode error while parsing file '{self.file}'.\n", file=sys.stderr)
            raise e
        except UnicodeDecodeError as e:
            print(f"\nUnicode decode error while parsing file '{self.file}'.\n", file=sys.stderr)
            raise e
        except Exception as e:
            print(f"\nException error processing file '{self.file}'.\n", file=sys.stderr)
            raise e

    def toml_read(self) -> object:
        try:
            return toml.load(self.file)
        except TomlDecodeError as e:
            print(f"\nTOML decode error while parsing file '{self.file}'.\n", file=sys.stderr)
            raise e
        except UnicodeDecodeError as e:
            print(f"\nUnicode decode error while parsing file '{self.file}'.\n", file=sys.stderr)
            raise e
        except Exception as e:
            print(f"\nException error processing file '{self.file}'.\n", file=sys.stderr)
            raise e

    def xlsx_read(self) -> object:
        try:
            return pd.read_excel(self.file, sheet_name=None)
        except XLRDError as e:
            print(f"\nXLSX XLRD error while parsing file '{self.file}'.\n", file=sys.stderr)
            raise e
        except Exception as e:
            print(f"\nException error processing file '{self.file}'.\n", file=sys.stderr)
            raise e

    def xml_read(self) -> object:
        try:
            return etree.parse(os.fspath(self.file))
        except XMLSyntaxError as e:
            print(f"\nXML Syntax error while parsing file '{self.file}'.\n", file=sys.stderr)
            raise e
        except Exception as e:
            print(f"\nException error processing file '{self.file}'.\n", file=sys.stderr)
            raise e

    def yaml_read(self) -> object:
        try:
            with open(self.file, 'r') as open_file:
                return yaml.safe_load(open_file)
        except ParserError as e:
            print(f"\nYAML parse error while parsing file '{self.file}'.\n", file=sys.stderr)
            raise e
        except ScannerError as e:
            print(f"\nYAML Scanner error while parsing file '{self.file}'.\n", file=sys.stderr)
            raise e
        except UnicodeDecodeError as e:
            print(f"\nUnicode decode error while parsing file '{self.file}'.\n", file=sys.stderr)
            raise e
        except Exception as e:
            print(f"\nException error processing file '{self.file}'.\n", file=sys.stderr)
            raise e



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# Do NOT edit this system file by hand -- use git.  See "URL to git source" below.
#
# Author:        $Id: Thomas R. Stevenson <aa0026@wayne.edu> $
#
# Last Changed:  $Date: Tue Feb 21 12:10:36 2023 -0500 $
#
# URL to git source: $URL: git@git.wayne.edu:ECS_Projects/ECSpylibs.git $
#

# Futuristic implementation of commands.  Must be first from/import
# command.

from __future__ import print_function

# Python Standard libraries.
import sys
from dataclasses import dataclass

import autologging
import base64
import logging
import pickle

from Crypto.Cipher import AES
from autologging import logged, traced
from os import urandom


@traced
@logged
@dataclass
class Password:
    """Add, Delete, or return User ID and Password information stored in an application PW file."""
    password_file: str = None

    def __post_init__(self):
        """Set up the Password environment."""

        self.BLOCK_SIZE = 32
        self.PADDING = "{"

        try:
            self.data = pickle.load(open(self.password_file, "rb"))
            self.secret = self.data["secret"]
            self.passwd = self.data["passwd"]
            self.__log.debug("Successfully read the password file '%s'.", self.password_file)
        except Exception as e:
            self.secret = urandom(self.BLOCK_SIZE)
            self.passwd = {}
            self.data = {"secret": self.secret, "passwd": self.passwd}
            self.__log.warning("Password file, '%s', is missing or empty.", self.password_file)
            self.__log.warning("Creating new password file '%s'.", self.password_file)
            self.__log.exception("Exception: '%s'", e)

        self.cipher = AES.new(self.secret)

    def pad(self, passwd: object) -> str:
        """Passwords must be a fixed length, so pad short passwords to the correct len."""

        return passwd + (self.BLOCK_SIZE - len(passwd) % self.BLOCK_SIZE) * self.PADDING

    def update(self) -> bool:
        """Update the PW file."""

        self.data = {"secret": self.secret, "passwd": self.passwd}
        pickle.dump(self.data, open(self.password_file, "wb"))
        return True

    def set(self, user_id: object, passwd: object) -> bool:
        """Add/Modify a User/PW entry in the PW file."""

        self.passwd[user_id] = base64.b64encode(self.cipher.encrypt(self.pad(passwd)))
        self.update()
        return True

    def get(self, user_id: object) -> str:
        """Verify and return, if exists, the PW for the given ID."""

        if user_id in self.passwd:
            user_pw = self.cipher.decrypt(base64.b64decode(self.passwd[user_id])).decode("utf-8").rstrip(self.PADDING)
        else:
            user_pw = ""

        return user_pw

    def delete(self, user_id: object) -> bool:
        """Verify and delete a ID/PW entry in the PW file."""

        if user_id in self.passwd:
            self.__log.warning("Deleting User id '%s'.", user_id)
            del self.passwd[user_id]
            self.update()
        else:
            self.__log.warning("Id '%s' does not exists.", user_id)

        return True

    def list_ids(self) -> bool:
        """Generate a list of User ID in the PW file."""

        print(f"ID", file=sys.stdout)
        user_ids = list(self.passwd)
        user_ids.sort()
        for user_id in user_ids:
            print(f"{user_id:<10}", file=sys.stdout)

        return True

    def list_pws(self) -> bool:
        """Generate a list of User ID and Passwords in the PW file."""

        print(f"{'ID':<15} Password", file=sys.stdout)
        user_ids = list(self.passwd)
        user_ids.sort()
        for user_id in user_ids:
            user_pw = self.get(user_id)
            print(f"{user_id:<15} {user_pw}", file=sys.stdout)

        return True



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['ecspylibs']

package_data = \
{'': ['*']}

setup_kwargs = {
    'name': 'ecspylibs',
    'version': '1.1.27',
    'description': 'Local python libraries used across multiple programs.',
    'long_description': '# ECSpylibs\n\n# Package files\n    -rw-r--r--. 1 tom tom 2568 Feb  2 13:57 src/ecspylibs/buildfunctionlist.py\n    -rw-r--r--. 1 tom tom 8390 Feb  2 13:57 src/ecspylibs/checkservice.py\n    -rw-r--r--. 1 tom tom 3090 Feb  2 13:57 src/ecspylibs/cleanuptempdirs.py\n    -rw-r--r--. 1 tom tom  465 Jan 26 11:05 src/ecspylibs/__init__.py\n    -rw-r--r--. 1 tom tom 3678 Feb  2 13:57 src/ecspylibs/initsetup.py\n    -rw-r--r--. 1 tom tom 1676 Feb  8 12:50 src/ecspylibs/parallel.py\n    -rw-r--r--. 1 tom tom 4965 Feb  2 13:57 src/ecspylibs/parseemail.py\n    -rw-r--r--. 1 tom tom 3644 Feb  2 13:57 src/ecspylibs/parsexml.py\n    -rw-r--r--. 1 tom tom 3650 Feb  2 13:57 src/ecspylibs/password.py\n    -rw-r--r--. 1 tom tom 1879 Feb  2 13:57 src/ecspylibs/reapchildren.py\n    -rw-r--r--. 1 tom tom 2993 Feb  2 13:57 src/ecspylibs/sendemail.py\n\n# Git information\n[GIT Home][ECSPYLIBS],\n[README File][README],\n[LICENSE File][LICENSE],\n[ChangeLog File][CHANGELOG],\n[pyproject.toml File][PYPROJECT].\n\n# About Me\n[My contact information][About Me].\n\n[ECSPYLIBS]: https://git.wayne.edu/ECS_Projects/ECSpylibs\n[README]: https://git.wayne.edu/ECS_Projects/ECSpylibs/-/blob/master/README.md\n[LICENSE]: https://git.wayne.edu/ECS_Projects/ECSpylibs/-/blob/master/LICENSE.txt\n[CHANGELOG]: https://git.wayne.edu/ECS_Projects/ECSpylibs/-/blob/master/ChangeLog\n[PYPROJECT]: https://git.wayne.edu/ECS_Projects/ECSpylibs/-/blob/master/pyproject.toml\n[About Me]: https://About.Me/Thomas.R.Stevenson\n',
    'author': 'Thomas R. Stevenson',
    'author_email': 'aa0026@wayne.edu',
    'maintainer': 'Thomas R. Stevenson',
    'maintainer_email': 'aa0026@wayne.edu',
    'url': 'https://git.wayne.edu/ECS_Projects/ECSpylibs.git',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'python_requires': '>=3.9,<4.0',
}


setup(**setup_kwargs)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Do NOT edit this system file by hand -- use git.
# See "URL to git source" below.
#
# Author:        $Id: Thomas R. Stevenson <aa0026@wayne.edu> $
#
# Last Changed:  $Date: Tue Feb 21 12:10:36 2023 -0500 $
#
# URL to git source: $URL: git@git.wayne.edu:ECS_Projects/ECSpylibs.git $
#

"""
blah
"""

import time
from dataclasses import dataclass

from autologging import logged, traced


@traced
@logged
@dataclass
class Parallel:
    """
    Blah
    """
    process: object
    pool_executor: object
    pool_size: object = None

    def __post_init__(self):
        """
        Blah
        """

        self.run_time = None
        self.results = None
        self.arguments = None

        if self.pool_size:
            if isinstance(self.pool_size, int) or self.pool_size.isnumeric():
                self.pool_size = int(self.pool_size)
            else:
                self.pool_size = None
        else:
            self.pool_size = None

        self.__log.debug(f"Process       : {type(self.process)}")
        self.__log.debug(f"Pool Executor : {self.pool_executor}")
        self.__log.debug(f"Pool Size     : {self.pool_size}")

    def run(self, arguments: object) -> object:
        """
        Blah
        """

        self.arguments = arguments

        self.__log.debug(f"arguments   : {self.arguments}")

        start_time = time.perf_counter()

        with self.pool_executor(self.pool_size) as executor:
            self.results = executor.map(self.process, self.arguments)

        finish_time = time.perf_counter()

        self.run_time = finish_time - start_time

        return self.results



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# Do NOT edit this system file by hand -- use git.  See "URL to git source" below.
#
# Author:        $Id: Thomas R. Stevenson <aa0026@wayne.edu> $
#
# Last Changed:  $Date: Fri Feb 17 12:47:46 2023 -0500 $
#
# URL to git source: $URL: git@git.wayne.edu:ECS_Projects/ECSpylibs.git $
#

# Futuristic implementation of commands.  Must be first from/import
# command.

from __future__ import print_function

# Home grown libraries.

from ecspylibs.password import Password

# Python Standard libraries.

import autologging
import logging
import requests
import sys

from autologging import logged, traced


@traced
@logged
class CheckService:
    """Verify if a given service has status up, maintenance, or is invalid."""

    def __init__(self, cherwell_user: object, passwd_file: object) -> None:
        """Setup the CheckService environment."""

        self.results_count = None
        self.check_service_url = None
        self.service_name = None
        self.cherwell_user = cherwell_user
        self.passwd_file = passwd_file
        self.count = 0

        self.cherwell_pw_data = Password(self.passwd_file)

        self.cherwell_pw = self.cherwell_pw_data.get(self.cherwell_user)
        self.client_Id = self.cherwell_pw_data.get("client_Id")
        self.bus_obj_Id = self.cherwell_pw_data.get("bus_obj_Id")
        self.search_Id = self.cherwell_pw_data.get("search_Id")

        del self.cherwell_pw_data

        self.__log.debug("User            : '%s'", self.cherwell_user)
        self.__log.debug("Pass            : '%s'", self.cherwell_pw)
        self.__log.debug("bus_obj_Id      : '%s'", self.bus_obj_Id)
        self.__log.debug("client_Id       : '%s'", self.client_Id)
        self.__log.debug("search_Id       : '%s'", self.search_Id)

        self.get_token = {
            "grant_type": "password",
            "client_id": self.client_Id,
            "username": self.cherwell_user,
            "password": self.cherwell_pw,
        }

        del self.cherwell_pw

        # There are two types of Cherwell searches: searchid and searchname.
        # Create both and then set self.search_API to the one being used.

        self.cherwell_fields_search_id = {
            "URI": "https://csm.wayne.edu/CherwellAPI",
            "API": "api/V1/getsearchresults/association",
            "busObjId": self.bus_obj_Id,
            "scope": "scope/Global",
            "owner": "scopeowner/(None)",
            "search": "searchid/%s?searchTerm=%%(searchTerm)s" % self.search_Id,
        }

        self.cherwell_fields_search_name = {
            "URI": "https://csm.wayne.edu/CherwellAPI",
            "API": "api/V1/getsearchresults/association",
            "busObjId": self.bus_obj_Id,
            "scope": "scope/Global",
            "owner": "scopeowner/(None)",
            "search": "searchname/HappeningNow%(searchTerm)s"
        }

        self.login_api = "%(URI)s/token?auth_mode=Internal" % self.cherwell_fields_search_id

        self.search_name_api = "%(URI)s/%(API)s/%(busObjId)s/%(scope)s/%(owner)s/%(search)s" % self.cherwell_fields_search_name
        self.search_id_API = "%(URI)s/%(API)s/%(busObjId)s/%(scope)s/%(owner)s/%(search)s" % self.cherwell_fields_search_id

        # Set self.search_API to the one being used from above.

        self.search_api = self.search_name_api

        self.__log.debug("Login API       : '%s'", self.login_api)
        self.__log.debug("Search Name API : '%s'", self.search_name_api)
        self.__log.debug("Search ID API   : '%s'", self.search_id_API)
        self.__log.debug("Search API      : '%s'", self.search_api)

        for self.i in range(1, 9):
            try:
                self.__log.debug("Call %d login_API.", self.i)
                self.auth_r = requests.post(self.login_api, data=self.get_token)
                self.__log.debug("Call %d to login_API successful.", self.i)
                break
            except Exception as e:
                self.__log.critical("Error attempt number %d from call to login_API.", self.i)
                self.__log.exception("Exception: '%s'.", e)
        else:
            self.__log.error("Call to login_API unsuccessful.")
            self.external_errors = True
            self.internal_errors = True
            return

        for self.i in range(1, 9):
            try:
                self.__log.debug("Call %d raise_for_status.", self.i)
                self.auth_r.raise_for_status()
                self.__log.debug("Call %d to raise_for_status successful.", self.i)
                break
            except Exception as e:
                self.__log.critical("Error attempt number %d from self.auth_r.raise_for_status().", self.i)
                self.__log.exception("Exception: '%s'.", e)
        else:
            self.__log.error("Call to raise_for_status unsuccessful.")
            self.external_errors = True
            self.internal_errors = True
            return

        if "access_token" in self.auth_r.json():
            self.token = self.auth_r.json()["access_token"]
        else:
            self.__log.critical('self.auth_r.json()["access_token"] is missing or invalid.')
            self.external_errors = True
            self.internal_errors = True
            return

        if "token_type" in self.auth_r.json():
            self.token_type = self.auth_r.json()["token_type"]
        else:
            self.__log.critical('self.auth_r.json()["token_type"] is missing or invalid.')
            self.external_errors = True
            self.internal_errors = True
            return

        self.auth_header = {
            "Authorization": "%s %s" % (self.token_type, self.token),
            "Accept": "application/json",
        }

    def check_service(self, service_name: object) -> object:
        """Verify if a given service has status up, maintenance, or is invalid.

    Return codes:
        0 : Service should be up.  Continue with check.
        1 : Service is in maintenance.  Skip check.
        2 : Error calling Cherwell.  Check error log for details.
        3 : Error returning service status from Cherwell.  Check error log for details.
        4 : Error processing service status retrieved from Cherwell.  Check error log for details.
    """

        self.service_name = service_name
        self.check_service_url = self.search_api % {"searchTerm": self.service_name}

        self.__log.debug("Service Name = '%s'", self.service_name)
        self.__log.debug("URL = '%s'", self.check_service_url)

        for self.i in range(1, 9):
            try:
                self.__log.debug("Call %d self.check_service_URL.", self.i)
                self.r = requests.get(self.check_service_url, headers=self.auth_header)
                self.__log.debug("Call %d to self.check_service_URL successful.", self.i)
                break
            except Exception as e:
                self.__log.critical(
                    "Error attempt number %d from requests.get(self.check_service_URL, headers=self.auth_header).",
                    self.i)
                self.__log.exception("Exception: '%s'.", e)
        else:
            self.__log.error("Call to self.check_service_URL unsuccessful.")
            self.external_errors = True
            self.internal_errors = True
            return 2

        for self.i in range(1, 9):
            try:
                self.__log.debug("Call %d to self.r.raise_for_status.", self.i)
                self.r.raise_for_status()
                self.__log.debug("Call %d to self.r.raise_for_status successful.", self.i)
                break
            except Exception as e:
                self.__log.critical("Error attempt number %d from self.r.raise_for_status().", self.i)
                self.__log.exception("Exception: '%s'.", e)
        else:
            self.__log.error("Call to self.r.raise_for_status unsuccessful.")
            self.external_errors = True
            self.internal_errors = True
            return 3

        if "totalRows" in self.r.json():
            self.results_count = self.r.json()["totalRows"]
            self.__log.debug("Count = '%s'", self.results_count)
        else:
            self.__log.critical('self.r.json()["totalRows"] is missing or invalid.')
            self.external_errors = True
            self.internal_errors = True
            return 4

        return self.results_count
